
#include <stdio.h>
#include <stdlib.h>
#include <pwd.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <unistd.h>
#include <qfiledialog.h>
#include <math.h>
#include <qvgroupbox.h>
#include <qvbox.h>
#include <qhgroupbox.h>
#include <kglobal.h>
#include <qpushbutton.h>
#include <kicontheme.h>
#include <kiconloader.h>
#include <qmessagebox.h>
#include <qtimer.h>
#include <qgrid.h>
#include <klocale.h>
#include <kconfig.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>

#include "misc.h"
#include "userlist.h"

UserList userlist;
struct config config;

char *preparePath(char* filename)
{
	static char path[1023];
	char *home;
	struct passwd* pw;
	if ((pw = getpwuid(getuid())) != NULL)
		home = pw->pw_dir;
	else
		home = getenv("HOME");
	snprintf(path, sizeof(path), "%s/.gg/%s", home, filename);
	return path;
};

UinsList::UinsList() {
}

bool UinsList::equals(UinsList &uins) {
	if (count() != uins.count())
		return false;
	for (UinsList::iterator i = begin(); i != end(); i++)
		if(!uins.contains(*i))
			return false;
	return true;
}

void UinsList::sort() {
	bool stop;
	int i, Count;
	uin_t uin;
	
	Count = count();
	if (Count < 2)
		return;

	do {
		stop = true;
		for (i = 0; i < Count - 1; i++)
			if (this->operator[](i) > this->operator[](i+1)) {
				uin = this->operator[](i);
				this->operator[](i) = this->operator[](i+1);
				this->operator[](i+1) = uin;
				stop = false;
				}
	} while (!stop);	
}

void cp_to_iso(unsigned char *buf) {
    while (*buf) {
	if (*buf == (unsigned char)'') *buf = '';
        if (*buf == (unsigned char)'') *buf = '';
        if (*buf == 140) *buf = '';
        if (*buf == 156) *buf = '';
        if (*buf == 143) *buf = '';
        if (*buf == 159) *buf = '';

        buf++;
        }
}

void iso_to_cp(unsigned char *buf) {
    while (*buf) {
	if (*buf == (unsigned char)'') *buf = '';
        if (*buf == (unsigned char)'') *buf = '';
        if (*buf == (unsigned char)'') *buf = 140;
        if (*buf == (unsigned char)'') *buf = 156;
        if (*buf == (unsigned char)'') *buf = 143;
        if (*buf == (unsigned char)'') *buf = 159;

        buf++;
        }
}

void loadKaduConfig(void) {  	
	fprintf(stderr, "KK loadKaduConfig(): Touching config file...\n");

	KConfig *konf;
	konf = new KConfig(preparePath("kadu.conf"));

	konf->setGroup("Global");
	config.uin = konf->readNumEntry("UIN",0);
	config.password = pwHash(konf->readEntry("Password",""));
	fprintf(stderr,"KK Read user data: uin %d password %s\n",config.uin,config.password);
	config.soundprog = strdup(konf->readEntry("SoundPlayer",""));
	config.soundmsg = strdup(konf->readEntry("Message_sound",""));
	config.soundvolctrl = konf->readBoolEntry("VolumeControl",false);
	config.soundvol = konf->readDoubleNumEntry("SoundVolume",1.0);
	config.soundchat = strdup(konf->readEntry("Chat_sound",""));
	config.nick = konf->readEntry("Nick", i18n("Me"));
	
	config.defaultstatus = konf->readNumEntry("DefaultStatus", GG_STATUS_NOT_AVAIL);

	if (!config.defaultstatus)
		config.defaultstatus = GG_STATUS_NOT_AVAIL;
	config.defaultdescription = konf->readEntry("DefaultDescription", i18n("I am busy."));

	config.logmessages = konf->readBoolEntry("Logging",true);
	config.savegeometry = konf->readBoolEntry("SaveGeometry",true);
	config.playsoundchat = konf->readBoolEntry("PlaySoundChat",true);
	config.playsoundchatinvisible = konf->readBoolEntry("PlaySoundChatInvisible",true);
	config.playsound = konf->readBoolEntry("PlaySound",false);
	config.playartsdsp = konf->readBoolEntry("PlaySoundArtsDsp",false);
	config.sysmsgidx = konf->readNumEntry("SystemMsgIndex",0);
	config.autoaway = konf->readBoolEntry("AutoAway", false);
	config.autoawaytime = konf->readNumEntry("AutoAwayTime", 300);
	config.allowdcc = konf->readBoolEntry("AllowDCC",false);
	config.dccip = strdup(konf->readEntry("DccIP", "0.0.0.0"));
	config.extip = strdup(konf->readEntry("ExternalIP", "0.0.0.0"));
	config.extport = konf->readNumEntry("ExternalPort", 0);
	config.server = strdup(konf->readEntry("Server", "0.0.0.0"));
	config.dock = konf->readBoolEntry("UseDocking",true);
	config.raise = konf->readBoolEntry("AutoRaise",false);
	config.privatestatus = konf->readBoolEntry("PrivateStatus", false);
	config.rundocked = konf->readBoolEntry("RunDocked", false);
	config.grouptabs = konf->readBoolEntry("DisplayGroupTabs",true);

	if (config.savegeometry)
		config.geometry = konf->readRectEntry("Geometry");

	konf->setGroup("SMS");
	config.smsapp = strdup(konf->readEntry("SmsApp",""));
	config.smscustomconf = konf->readBoolEntry("UseCustomString",false);
	config.smsconf = strdup(konf->readEntry("SmsString",""));

	konf->setGroup("Other");
	config.emoticons = konf->readBoolEntry("UseEmoticons",false);
	config.autosend = konf->readBoolEntry("AutoSend",false);
	config.scrolldown = konf->readBoolEntry("ScrollDown",true);
	config.emoticonspath = strdup(konf->readEntry("EmoticonsPath",""));
	config.chatprune = konf->readBoolEntry("ChatPrune",false);
	config.chatprunelen = konf->readNumEntry("ChatPruneLen",20);
	config.msgacks = konf->readBoolEntry("MessageAcks", true);
	
	konf->setGroup("Notify");
	config.soundnotify = strdup(konf->readEntry("NotifySound",""));
	config.notifyglobal = konf->readBoolEntry("NotifyStatusChange",false);
	config.notifydialog = konf->readBoolEntry("NotifyWithDialogBox",false);
	config.notifysound = konf->readBoolEntry("NotifyWithSound",false);
	config.notifies = konf->readListEntry("NotifyUsers");

	konf->setGroup("Proxy");
	config.useproxy = konf->readBoolEntry("UseProxy",false);
	config.proxyaddr = strdup(konf->readEntry("ProxyHost",""));
	config.proxyport = konf->readNumEntry("ProxyPort",0);

	konf->setGroup("Colors");
	config.colors.userboxBgR = konf->readNumEntry("UserboxBgR",255);
	config.colors.userboxBgG = konf->readNumEntry("UserboxBgG",255);
	config.colors.userboxBgB = konf->readNumEntry("UserboxBgB",255);
	config.colors.userboxFgR = konf->readNumEntry("UserboxFgR",0);
	config.colors.userboxFgG = konf->readNumEntry("UserboxFgG",0);
	config.colors.userboxFgB = konf->readNumEntry("UserboxFgB",0);
	config.userboxFont = konf->readEntry("UserboxFont","sans-serif");
	config.userboxFontSize = konf->readNumEntry("UserboxFontSize",12);
	config.chatFont = konf->readEntry("ChatFont","sans-serif");
	config.chatFontSize = konf->readNumEntry("ChatFontSize",12);

	fprintf(stderr,"KK loadKaduConfig(): finished.\n");

	delete konf;
}

void saveKaduConfig(void) {
	fprintf(stderr,"KK saveKaduConfig(): Writing config files...\n");
	KConfig * konf;
	konf = new KConfig(preparePath("kadu.conf"));

	konf->setGroup("Global");
	konf->writeEntry("UIN",config.uin);
	konf->writeEntry("Password",pwHash(config.password));
	konf->writeEntry("Nick", config.nick);
//	konf->writeEntry("Geometry",kadu->geometry());
	konf->writeEntry("Message_sound",config.soundmsg);
	konf->writeEntry("Chat_sound",config.soundchat);
	konf->writeEntry("Logging",config.logmessages);
	konf->writeEntry("DefaultStatus",config.defaultstatus);
	konf->writeEntry("DefaultDescription", config.defaultdescription);
	konf->writeEntry("SystemMsgIndex",config.sysmsgidx);
	konf->writeEntry("SaveGeometry",config.savegeometry);
	konf->writeEntry("PlaySoundChat",config.playsoundchat);
	konf->writeEntry("PlaySoundChatInvisible",config.playsoundchatinvisible);
	konf->writeEntry("SoundPlayer",config.soundprog);
	konf->writeEntry("PlaySound",config.playsound);
	konf->writeEntry("PlaySoundArtsDsp",config.playartsdsp);
	konf->writeEntry("SoundVolume",config.soundvol);
	konf->writeEntry("VolumeControl",config.soundvolctrl);
	konf->writeEntry("AutoAway",config.autoaway);
	konf->writeEntry("AutoAwayTime",config.autoawaytime);
	konf->writeEntry("AllowDCC",config.allowdcc);
	konf->writeEntry("DccIP", config.dccip);
	konf->writeEntry("ExternalIP", config.extip);
	konf->writeEntry("ExternalPort", config.extport);
	konf->writeEntry("Server", config.server);
	konf->writeEntry("UseDocking",config.dock);
	konf->writeEntry("AutoRaise",config.raise);
	konf->writeEntry("PrivateStatus",config.privatestatus);
	konf->writeEntry("RunDocked",config.rundocked);
	konf->writeEntry("DisplayGroupTabs",config.grouptabs);

	konf->setGroup("SMS");
	konf->writeEntry("SmsApp",config.smsapp);
	konf->writeEntry("SmsString",config.smsconf);
	konf->writeEntry("UseCustomString",config.smscustomconf);

	konf->setGroup("Other");
	konf->writeEntry("UseEmoticons",config.emoticons);
	konf->writeEntry("AutoSend",config.autosend);
	konf->writeEntry("ScrollDown",config.scrolldown);
	konf->writeEntry("EmoticonsPath",config.emoticonspath);
	konf->writeEntry("ChatPrune",config.chatprune);
	konf->writeEntry("ChatPruneLen",config.chatprunelen);
	konf->writeEntry("MessageAcks", config.msgacks);
	
	konf->setGroup("Proxy");
	konf->writeEntry("UseProxy",config.useproxy);
	konf->writeEntry("ProxyHost",config.proxyaddr);
	konf->writeEntry("ProxyPort",config.proxyport);

	konf->setGroup("Notify");
	konf->writeEntry("NotifyUsers",config.notifies);
	konf->writeEntry("NotifySound",config.soundnotify);
	konf->writeEntry("NotifyStatusChange",config.notifyglobal);
	konf->writeEntry("NotifyWithDialogBox",config.notifydialog);
	konf->writeEntry("NotifyWithSound",config.notifysound);

	konf->setGroup("Colors");
	konf->writeEntry("UserboxBgR", config.colors.userboxBgR);
	konf->writeEntry("UserboxBgG", config.colors.userboxBgG);
	konf->writeEntry("UserboxBgB", config.colors.userboxBgB);
	konf->writeEntry("UserboxFgR", config.colors.userboxFgR);
	konf->writeEntry("UserboxFgG", config.colors.userboxFgG);
	konf->writeEntry("UserboxFgB", config.colors.userboxFgB);
	konf->writeEntry("UserboxFont", config.userboxFont);
	konf->writeEntry("UserboxFontSize", config.userboxFontSize);
	konf->writeEntry("ChatFont", config.chatFont);
	konf->writeEntry("ChatFontSize", config.chatFontSize);

	konf->sync();
	delete konf;
}

char *pwHash(const char *tekst) {
    char *nowytekst;
    nowytekst = strdup(tekst);
    int ile, znak;
    for (ile = 0; ile < int(strlen(tekst)); ile++) {
	znak = nowytekst[ile]^ile^1;
	nowytekst[ile] = znak;
	}
    return nowytekst;
}
